@@ -2,10 +2,7 @@
 # vim: ts=2 sw=2 expandtab
 
 package POE::Component::Client::DNS;
-{
-  $POE::Component::Client::DNS::VERSION = '1.053';
-}
-
+$POE::Component::Client::DNS::VERSION = '1.054';
 use strict;
 
 use Carp qw(croak);
@@ -280,7 +277,7 @@ sub _dns_do_request {
   $self->[SF_RESOLVER]->nameservers(
     @{ $req->{nameservers} || $self->[SF_NAMESERVERS] }
   );
-  my $resolver_socket = $self->[SF_RESOLVER]->bgsend(
+  my $resolver_ref = $self->[SF_RESOLVER]->bgsend(
     $req->{host},
     $req->{type},
     $req->{class}
@@ -288,12 +285,23 @@ sub _dns_do_request {
 
   # The request failed?  Attempt to retry.
 
-  unless ($resolver_socket) {
+  unless ($resolver_ref) {
     $remaining = 1 if $remaining > 1;
     $kernel->delay_add(send_request => $remaining, $req);
     return;
   }
 
+  my $resolver_socket;
+
+  if ( ref( $resolver_ref ) eq 'IO::Select' ) {
+    ($resolver_socket) = [ $resolver_ref->handles ]->[0][0];
+  }
+  else {
+    $resolver_socket = $resolver_ref;
+  }
+
+  $req->{RES_REF} = $resolver_ref;
+
   # Set a timeout for the request, and watch the response socket for
   # activity.
 
@@ -365,8 +373,10 @@ sub _dns_response {
   $kernel->delay($socket);
   $kernel->select_read($socket);
 
+  my $handle = delete $req->{RES_REF};
+
   # Read the DNS response.
-  my $packet = $self->[SF_RESOLVER]->bgread($socket);
+  my $packet = $self->[SF_RESOLVER]->bgread($handle);
 
   # Set the packet's answerfrom field, if the packet was received ok
   # and an answerfrom isn't already included.  This uses the
@@ -475,6 +485,10 @@ sub _check_hosts_file {
         "/etc/hosts",
       );
 
+      if ( $^O eq 'android' ) {
+        push @candidates, "/system/etc/hosts";
+      }
+      
       if ($^O eq "MSWin32" or $^O eq "Cygwin") {
         my $sys_dir;
         $sys_dir = $ENV{SystemRoot} || "c:\\Windows";
@@ -573,7 +587,7 @@ POE::Component::Client::DNS - non-blocking, parallel DNS client
 
 =head1 VERSION
 
-version 1.053
+version 1.054
 
 =head1 SYNOPSIS
 
